"""
A Python module to make Google Charts. This module is made by Siddhesh Chavan! The module's version is 1.0!
This module can create a Line chart, Column Chart, Area Chart, Bar Chart, Pie Chart, Table and multiple charts.
Import Statement:
    import pyhtmlchart

Documentation(Tutorial):
    For documentation go to https://sid72020123.github.io/pyhtmlchart/

History:
    24/05/2021(v0.0.0.1) --> First made the module.
    25/05/2021(v0.0.1) --> Made and updated the 'line_chart' package.
    26/05/2021(v0.1.5) --> Made the auto update structure in 'line_chart' package.
    27/05/2021(v0.2.9) --> Fixed many bugs.
    28/05/2021(v0.5.0) --> Made the 'multiple_charts' package and updated it.
    29/05/2021(v0.6.0) --> Made the 'column_chart' package and updated it.
    30/05/2021(v0.9.0) --> Made the 'area_chart','bar_chart', 'pie_chart' and 'table' packages and updated them.
    31/05/2021(v0.9.8) --> Fixed the color issue and updated all packages.
    01/06/2021(v0.9.8.5) --> Fixed the errors of 'multiple_charts' package.
    02/06/2021(v0.9.7) --> Fixed the errors of 'multiple_charts' package and added 'print_settings()' function.
    05/06/2021(v0.9.8) --> Added 'background_color' property to all the charts.
    06/06/2021(v0.9.9) --> Updated 'multiple_chart' package.
    13/06/2021(v1.0) --> First Release!
Credits:
    All code credits to Siddhesh Chavan
    Special thanks to Google Charts.
Information:
    Module made by:- Siddhesh Chavan
    Age:- 15 (as of 2021)
    Email:- siddheshchavan2020@gmail.com
    YouTube Channel:- Siddhesh Chavan (Link: https://www.youtube.com/channel/UCWcSxfT-SbqAktvGAsrtadQ)
    Scratch Account:- @Sid72020123 (Link: https://scratch.mit.edu/users/Sid72020123/)
    My self-made Website: https://Sid72020123.github.io/
"""
import pyhtmlchart.line_chart
import pyhtmlchart.column_chart
import pyhtmlchart.area_chart
import pyhtmlchart.bar_chart
import pyhtmlchart.pie_chart
import pyhtmlchart.table
import pyhtmlchart.multiple_charts

__name__ = "pyhtmlchart"
__version__ = 1.0
__documentation__ = "https://sid72020123.github.io/pyhtmlchart/"
__developer__ = "Siddhesh Chavan"
print(f"{__name__} v{__version__} - {__documentation__}")



"""
Package to display multiple charts on one '.html' page. To make a simple multiple chart, go to https://sid72020123.github.io/pyhtmlchart/multiple_charts.html
"""
import os


class MultipleCharts:
    def __init__(self, location, title="Multiple Chart", height=600, width=1000, frame_border=0, border=0,
                 border_color='Black'):
        """
        Class to create multiple charts on one page.
        :param location: The location of the '.html' page to be saved.
        :param title: The title of group of charts.
        :param height: The height of all frames.
        :param width: The width of all frames.
        :param frame_border: The frame border of all frames.
        :param border: The border width of all frames.
        :param border_color: The border color of all frames.
        """
        self.location = location
        self.title = title
        self.height = height
        self.width = width
        self.frame_border = frame_border
        self.border = border
        self.border_color = border_color
        self.frame_value = []
        self.frame_size = []
        self.chart_locations = ''

    def add_chart(self, chart_locations, frame_value=[], frame_size=[]):
        """
        Function to add the saved charts on one page.
        :param chart_locations: The list of all the chart's saved locations to be added on one page. The elements should be separated by commas and should be in 'string' format. Please give the correct location along with the extension. The order of charts in the page is same as the order of files given in the list. You can change the values in the list to add the charts accordingly.
        :param frame_value: If you want to change the size of one chart in a page then give the chart's list index according to the 'chart_locations' list and use the frame_size parameter to set a different size for that specified chart.
        :param frame_size: If you pass the 'frame_size' parameter, then give the size of that chart in a 2-d list. First is height and then width.
        """
        self.chart_locations = chart_locations
        self.frame_value = frame_value
        self.frame_size = frame_size
        frame_height = []
        frame_width = []
        for i in range(0, len(chart_locations)):
            frame_height.append(self.height)
            frame_width.append(self.width)

        for i in range(0, len(chart_locations)):
            for j in range(0, len(self.frame_value)):
                if i == self.frame_value[j]:
                    frame_height[i] = self.frame_size[j][0]
                    frame_width[i] = self.frame_size[j][1]

        file = open(f"{self.location}.html", "w")
        file.write(
            f"<html>\n<head>\n<title>\n{self.title}\n</title>\n</head>\n<body>\n<center>\n<h1>\n{self.title}\n</h1>\n</center>\n")
        for i in range(0, len(chart_locations)):
            file.write(
                f"<iframe src = '{chart_locations[i]}', width='{frame_width[i]}px', height = '{frame_height[i]}px', frameBorder='{self.frame_border}', style = 'border: {self.border}px solid {self.border_color}'>\n</iframe>\n")
        file.write("</body>\n</html>")
        file.close()

    def open(self):
        """
        Function to open the saved multiple charts.
        """
        os.system(f'start {self.location}.html')

    def print_settings(self):
        """
        Function to print the chart settings.
        """
        print(
            f"Multiple Chart - '{self.title}' settings:-\n\tChart Name:- {self.title}\n\tChart Type:- Multiple Chart\n\tChart Location:- {self.location}.html\n\tChart Height:- {self.height}\n\tChart Width:- {self.width}\n\tChart Border:- {self.border}\n\tFrame Border:- {self.frame_border}\n\tBorder Color:- {self.border_color}")
        print(f"\tSub charts:-")
        for i in range(0, len(self.chart_locations)):
            print(f"\t\t{i} -> {self.chart_locations[i]}")
        print(f"\tDifferent Size:- ")
        if len(self.frame_value) == 0:
            print('\t\tNone')
        else:
            for i in range(0, len(self.frame_value)):
                print(f"\t\t{self.frame_value[i]} -> {self.frame_size[i]}")



"""
Package to create column charts. To make a simple column chart, go to https://sid72020123.github.io/pyhtmlchart/#making-column-chart
"""
import os
import time


class ColumnChart:
    def __init__(self, location, title='Chart', background_color='White', legend_position='right', width=900,
                 height=500, chart_actions=False,
                 y_axis_title='Y', x_axis_title='X', color=None, column_width=20, print_log=False):
        """
        Class to make a Column Chart!
        :param location: The Location in which the chart is stored as a '.html' file.
        :param title: The Main Title of the Chart.
        :param background_color: The background color of the chart.
        :param legend_position: The position of the legend in the chart(use values like --> 'top', 'bottom', 'right')
        :param width: The width of the chart.
        :param height: The height of the chart.
        :param chart_actions: 'False' by default. Set it to 'True' to enable chart options to pan and zoom. Scroll mouse button for zoom and drag for pan.
        :param y_axis_title: The title of 'Y' axis.
        :param x_axis_title: The title of 'X' axis.
        :param color: The list of colors to be added to the chart. Make sure that it is a list and is in 'string' datatype. Set it to 'None' to get the default color.
        :param column_width: The width of the column in column chart.
        :param print_log: Set it to 'True' to see the process and 'False' to disable the log.
        """
        self.location = location
        self.title = title
        self.legend_position = legend_position
        self.width = width
        self.height = height
        self.data = []
        self.chart_actions = chart_actions
        self.data_titles = []
        self.y_axis_title = 'Y'
        self.x_axis_title = 'X'
        self.auto_update = False
        self.update_time = 5000
        self.y_axis_title = y_axis_title
        self.x_axis_title = x_axis_title
        self.color = color
        self.column_width = column_width
        self.wait = True
        self.print_log = print_log
        self.background_color = background_color

    def add_data(self, data, data_titles):
        """
        Add data to a Column Chart.
        :param data: The 2-d list of the data to be added.
        :param data_titles: The list of titles of the data. Make sure that the length of the titles list is equal to the length of a item in a 2-d list.
        """
        if self.print_log:
            print(f"{self.title}: Creating Chart.....")
        self.data_titles = data_titles
        self.data = data
        file = open(f'{self.location}.html', "w")
        file.write(f"<html>\n<head>\n<title>{self.title}</title>")
        file.write(
            f'\n<script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>\n<script type="text/javascript">')
        file.write("\ngoogle.charts.load('current', {packages: ['corechart', 'bar']});")
        file.write(
            "\ngoogle.charts.setOnLoadCallback(drawBasic);\nfunction drawBasic() {\nvar data = google.visualization.arrayToDataTable([\n")
        if self.print_log:
            print(f"{self.title}: Adding Data.....")
        file.write(f"{data_titles},\n")
        for i in range(0, len(self.data)):
            file.write(f"{self.data[i]},\n")
        if self.print_log:
            print(f"{self.title}: Data Added!")
        file.write("]);\n")
        file.write("var options = {")
        file.write("title: '" + str(
            self.title) + f"',\ncurveType: 'function',\n")
        if self.color is not None:
            file.write(f"\ncolors: {self.color},")
        file.write("bar: {groupWidth: '" + str(self.column_width) + "%'},\n")
        file.write("legend: { position: '" + str(self.legend_position) + "'},\n")
        file.write("vAxis: {\ntitle:" + "'" + str(self.y_axis_title) + "'},\n")
        file.write("hAxis: {\ntitle:" + "'" + str(self.x_axis_title) + "'},\n")
        file.write(f"backgroundColor: '{self.background_color}',\n")
        if self.chart_actions:
            file.write("explorer: {}\n")
        file.write("\n};")
        file.write(f"\nvar chart = new google.visualization.ColumnChart(document.getElementById('column_chart'));\n")
        file.write("chart.draw(data, options);\n}")
        file.write(
            f"</script>\n</head>\n<body>\n<div id = 'column_chart' style = 'width: {self.width}px; height: {self.height}px'></div>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"{self.title}: Chart Created!")

    def enable_auto_update(self, time=5000):
        """
        Function to enable auto update.
        :param time: The time of update(in millisecond)
        """
        if self.print_log:
            print(f"{self.title}: Enabling Auto Update.....")
        self.auto_update = True
        self.update_time = time
        file = open(f'{self.location}.html', "w")
        file.write(f"<html>\n<head>\n<title>{self.title}</title>")
        file.write(
            f'\n<script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>\n<script type="text/javascript">')
        file.write("\ngoogle.charts.load('current', {packages: ['corechart', 'bar']});")
        file.write(
            "\ngoogle.charts.setOnLoadCallback(drawBasic);\nfunction drawBasic() {\nvar data = google.visualization.arrayToDataTable([\n")
        file.write(f"{self.data_titles},\n")
        for i in range(0, len(self.data)):
            file.write(f"{self.data[i]},\n")
        file.write("]);\n")
        file.write("var options = {")
        file.write("title: '" + str(
            self.title) + f"',\ncurveType: 'function',\n")
        if self.color is not None:
            file.write(f"\ncolors: {self.color},")
        file.write("bar: {groupWidth: '" + str(self.column_width) + "%'},\n")
        file.write("legend: { position: '" + str(self.legend_position) + "'},\n")
        file.write("vAxis: {\ntitle:" + "'" + str(self.y_axis_title) + "'},\n")
        file.write("hAxis: {\ntitle:" + "'" + str(self.x_axis_title) + "'},\n")
        file.write(f"backgroundColor: '{self.background_color}',\n")
        if self.chart_actions:
            file.write("explorer: {}\n")
        file.write("\n};")
        file.write(f"\nvar chart = new google.visualization.ColumnChart(document.getElementById('column_chart'));\n")
        file.write("chart.draw(data, options);\n}\n")
        file.write('function AutoRefresh( t ) {\nsetTimeout("location.reload(true);", t);\n}')
        file.write(
            f"\n</script>\n</head>\n" + '<body onload = "JavaScript:AutoRefresh(' + str(
                self.update_time) + ');">' + f"\n<div id = 'column_chart' style = 'width: {self.width}px; height: {self.height}px'></div>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"{self.title}: Auto Update Enabled!")

    def disable_auto_update(self):
        """
        Function to disable the auto update.
        """
        if self.print_log:
            print(f"{self.title}: Disabling Auto Update.....")
        self.auto_update = False
        file = open(f'{self.location}.html', "w")
        file.write(f"<html>\n<head>\n<title>{self.title}</title>")
        file.write(
            f'\n<script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>\n<script type="text/javascript">')
        file.write("\ngoogle.charts.load('current', {packages: ['corechart', 'bar']});")
        file.write(
            "\ngoogle.charts.setOnLoadCallback(drawBasic);\nfunction drawBasic() {\nvar data = google.visualization.arrayToDataTable([\n")
        file.write(f"{self.data_titles},\n")
        for i in range(0, len(self.data)):
            file.write(f"{self.data[i]},\n")
        file.write("]);\n")
        file.write("var options = {")
        file.write("title: '" + str(
            self.title) + f"',\ncurveType: 'function',\n")
        if self.color is not None:
            file.write(f"\ncolors: {self.color},")
        file.write("bar: {groupWidth: '" + str(self.column_width) + "%'},\n")
        file.write("legend: { position: '" + str(self.legend_position) + "'},\n")
        file.write("vAxis: {\ntitle:" + "'" + str(self.y_axis_title) + "'},\n")
        file.write("hAxis: {\ntitle:" + "'" + str(self.x_axis_title) + "'},\n")
        file.write(f"backgroundColor: '{self.background_color}',\n")
        if self.chart_actions:
            file.write("explorer: {}\n")
        file.write("\n};")
        file.write(f"\nvar chart = new google.visualization.ColumnChart(document.getElementById('column_chart'));\n")
        file.write("chart.draw(data, options);\n}")
        file.write(
            f"</script>\n</head>\n<body>\n<div id = 'column_chart' style = 'width: {self.width}px; height: {self.height}px'></div>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"{self.title}: Auto Update Disabled!")

    def update_data(self, data, append=False, wait=True):
        """
        Function to update a data if auto update is enabled.
        :param data: The new data to be added.
        :param append: If 'False' then it it will not append the data to the chart. If 'True' the it will append the data to the chart.
        :param wait: Wait for a given update time if 'True' and disabled when 'False'.
        """
        if self.print_log:
            print(f"{self.title}: Updating Data.....")
        self.wait = wait
        if self.wait:
            time.sleep(self.update_time / 1000)
        self.auto_update = True
        if append:
            self.data += data
        else:
            self.data = data
        file = open(f'{self.location}.html', "w")
        file.write(f"<html>\n<head>\n<title>{self.title}</title>")
        file.write(
            f'\n<script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>\n<script type="text/javascript">')
        file.write("\ngoogle.charts.load('current', {packages: ['corechart', 'bar']});")
        file.write(
            "\ngoogle.charts.setOnLoadCallback(drawBasic);\nfunction drawBasic() {\nvar data = google.visualization.arrayToDataTable([\n")
        file.write(f"{self.data_titles},\n")
        for i in range(0, len(self.data)):
            file.write(f"{self.data[i]},\n")
        file.write("]);\n")
        file.write("var options = {")
        file.write("title: '" + str(
            self.title) + f"',\ncurveType: 'function',\n")
        if self.color is not None:
            file.write(f"\ncolors: {self.color},")
        file.write("bar: {groupWidth: '" + str(self.column_width) + "%'},\n")
        file.write("legend: { position: '" + str(self.legend_position) + "'},\n")
        file.write("vAxis: {\ntitle:" + "'" + str(self.y_axis_title) + "'},\n")
        file.write("hAxis: {\ntitle:" + "'" + str(self.x_axis_title) + "'},\n")
        file.write(f"backgroundColor: '{self.background_color}',\n")
        if self.chart_actions:
            file.write("explorer: {}\n")
        file.write("\n};")
        file.write(f"\nvar chart = new google.visualization.ColumnChart(document.getElementById('column_chart'));\n")
        file.write("chart.draw(data, options);\n}\n")
        file.write('function AutoRefresh( t ) {\nsetTimeout("location.reload(true);", t);\n}')
        file.write(
            f"\n</script>\n</head>\n" + '<body onload = "JavaScript:AutoRefresh(' + str(
                self.update_time) + ');">' + f"\n<div id = 'column_chart' style = 'width: {self.width}px; height: {self.height}px'></div>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"{self.title}: Data Updated!")

    def open(self):
        """
        Function to automatically open the chart in the browser. If the chart doesn't open then go to the saved location and open it.
        """
        if self.print_log:
            print(f"{self.title}: Opening chart.....")
        os.system(f'start {self.location}.html')
        if self.print_log:
            print(f"{self.title}: Chart Opened!")

    def print_chart_settings(self):
        """
        Function to print the Chart Settings.
        """
        chart_settings = f"Chart '{self.title}' Settings: \n\tChart Name:- {self.title}\n\tChart Type:- Column Chart\n\tChart Location:- {self.location}.html\n\tBackground Color:- {self.background_color}\n\tChart Width:- {self.width}\n\tChart Height:- {self.height}\n\tLegend Position:- {self.legend_position}\n\tChart Actions(pan/zoom):- {self.chart_actions}\n\tX Axis Title:- {self.x_axis_title}\n\tY Axis Title:- {self.y_axis_title}\n\tColor:- {self.color}\n\tColumn Width:- {self.column_width}\n\tAuto Update:- {self.auto_update}"
        print(chart_settings)
        if self.auto_update:
            print(f"\tUpdate Time:- {self.update_time} millisecond")
        if self.wait:
            print(f"\tWait for update:- {self.wait}")
        print(f"\tPrint Log:- {self.print_log}")



"""
Package to create Tables from given data. To make a simple table, go to https://sid72020123.github.io/pyhtmlchart/#making-table
"""
import os
import time


class Table:
    def __init__(self, location, title='Table', table_border=5, border_color='Black', cell_padding=3, cell_spacing=0,
                 cell_horizontal_text_align='center', cell_vertical_text_align='middle', column_bg_color='Yellow',
                 data_bg_color='White', column_text_color='Black', data_text_color='Black', print_log=False):
        """
        Class to Create a Table from given data.
        :param location: The location of the table to be stored as a '.html' file.
        :param title: The Title of the Table.
        :param table_border: The Border of the table.
        :param border_color: The Border color of the table.
        :param cell_padding: The cell padding. The space between the cell contents and cell border.
        :param cell_spacing: The cell spacing. The space between the cell and table border.
        :param cell_horizontal_text_align: The horizontal align of the text in the table. The values are: 'left','center','right'.
        :param cell_vertical_text_align: The vertical align of the text in the table. The values are: 'top','bottom','center'.
        :param column_bg_color: The Background color of the column.
        :param data_bg_color: The Background color of the data.
        :param column_text_color: The text color of the column.
        :param data_text_color: The text color of the data.
        :param print_log: Set it to 'True' to see the log of making of the table.
        """
        self.location = location
        self.title = title
        self.data = []
        self.columns = []
        self.table_border = table_border
        self.border_color = border_color
        self.cell_padding = cell_padding
        self.cell_spacing = cell_spacing
        self.cell_horizontal_text_align = cell_horizontal_text_align
        self.cell_vertical_text_align = cell_vertical_text_align
        self.column_bg_color = column_bg_color
        self.data_bg_color = data_bg_color
        self.column_text_color = column_text_color
        self.data_text_color = data_text_color
        self.auto_update = False
        self.update_time = 5000
        self.wait = True
        self.print_log = print_log

    def add_data(self, data, columns):
        """
        Function to add the data to the table.
        :param data: The 2-d list of all data to be added.
        :param columns:  The 2-d list of all column data to be added. Make sure that the length of the data is equal to the length of the column list.
        """
        self.data = data
        self.columns = columns
        if self.print_log:
            print(f"'{self.title}': Creating Table.....")
        file = open(f'{self.location}.html', 'w')
        file.write(f"<html>\n<head>\n<title>\n{self.title}\n</title>\n</head>\n<body>\n<center><h1>{self.title}</h1>\n")
        file.write(
            f"<table border = '{self.table_border}' BorderColor='{self.border_color}' cellpadding='{self.cell_padding}' cellspacing='{self.cell_spacing}'>\n")
        file.write(
            f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.column_bg_color}' style='color: {self.column_text_color};'>\n")
        if self.print_log:
            print(f"'{self.title}': Adding Data.....")
        for i in range(0, len(columns)):
            file.write(
                f"<td align='{self.cell_horizontal_text_align}'>{columns[i]}</td>\n")
        file.write(f"</tr>\n")

        for i in range(0, len(data)):
            file.write(
                f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.data_bg_color}' style='color: {self.data_text_color};'>\n")
            for j in range(0, len(data[i])):
                file.write(
                    f"<td align='{self.cell_horizontal_text_align}'>{data[i][j]}</td>\n")
            file.write(f"</tr>\n")
        file.write(f"</table>\n</center>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"'{self.title}': Created Table!")

    def enable_auto_update(self, time=5000):
        """
        Function to enable auto update for a Table.
        :param time: The time  interval to update the data (in millisecond).
        """
        self.auto_update = True
        self.update_time = time
        if self.print_log:
            print(f"'{self.title}': Enabling Auto Update.....")
        file = open(f'{self.location}.html', 'w')
        file.write(
            f"<html>\n<head>\n<title>\n{self.title}\n</title>\n</head>\n<body onload='AutoRefresh({self.update_time})'>\n<center><h1>{self.title}</h1>\n")
        file.write(
            f"<table border = '{self.table_border}' BorderColor='{self.border_color}' cellpadding='{self.cell_padding}' cellspacing='{self.cell_spacing}'>\n")
        file.write(
            f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.column_bg_color}' style='color: {self.column_text_color};'>\n")
        for i in range(0, len(self.columns)):
            file.write(
                f"<td align='{self.cell_horizontal_text_align}'>{self.columns[i]}</td>\n")
        file.write(f"</tr>\n")

        for i in range(0, len(self.data)):
            file.write(
                f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.data_bg_color}' style='color: {self.data_text_color};'>\n")
            for j in range(0, len(self.data[i])):
                file.write(
                    f"<td align='{self.cell_horizontal_text_align}'>{self.data[i][j]}</td>\n")
            file.write(f"</tr>\n")
        file.write(f"</table>\n</center>\n</body>\n")
        file.write('<script>\nfunction AutoRefresh(t){\nsetInterval("location.reload(true);",' +
                   str(self.update_time) + ')\n}\n</script>')
        file.write("\n</html>")
        file.close()
        if self.print_log:
            print(f"'{self.title}': Auto Update Enabled!")

    def disable_auto_update(self):
        """
        Function to disable auto update to the Table.
        """
        self.auto_update = False
        if self.print_log:
            print(f"'{self.title}': Disabling Auto Update.....")
        file = open(f'{self.location}.html', 'w')
        file.write(f"<html>\n<head>\n<title>\n{self.title}\n</title>\n</head>\n<body>\n<center><h1>{self.title}</h1>\n")
        file.write(
            f"<table border = '{self.table_border}' BorderColor='{self.border_color}' cellpadding='{self.cell_padding}' cellspacing='{self.cell_spacing}'>\n")
        file.write(
            f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.column_bg_color}' style='color: {self.column_text_color};'>\n")
        for i in range(0, len(self.columns)):
            file.write(
                f"<td align='{self.cell_horizontal_text_align}'>{self.columns[i]}</td>\n")
        file.write(f"</tr>\n")

        for i in range(0, len(self.data)):
            file.write(
                f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.data_bg_color}' style='color: {self.data_text_color};'>\n")
            for j in range(0, len(self.data[i])):
                file.write(
                    f"<td align='{self.cell_horizontal_text_align}'>{self.data[i][j]}</td>\n")
            file.write(f"</tr>\n")
        file.write(f"</table>\n</center>\n</body>\n</html>")
        file.close()
        if self.print_log:
            print(f"'{self.title}': Auto Update Disabled!")

    def update_data(self, data, append=False, wait=True):
        """
        Function to update the data of a Table.
        :param data: The list of data to be updated.
        :param append:  If 'True' then it appends the new data with the previous data and shows the Table.
        :param wait: Wait for the data to update. Set it to 'True' to wait for the given time and then update the data or else set it to 'False' to update the data continuously.
        """
        self.wait = wait
        self.auto_update = True
        if self.print_log:
            print(f"'{self.title}': Updating Data.....")
        if append:
            self.data += data
        else:
            self.data = data
        if self.wait:
            time.sleep(self.update_time / 1000)
        file = open(f'{self.location}.html', 'w')
        file.write(
            f"<html>\n<head>\n<title>\n{self.title}\n</title>\n</head>\n<body onload='AutoRefresh({self.update_time})'>\n<center><h1>{self.title}</h1>\n")
        file.write(
            f"<table border = '{self.table_border}' BorderColor='{self.border_color}' cellpadding='{self.cell_padding}' cellspacing='{self.cell_spacing}'>\n")
        file.write(
            f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.column_bg_color}' style='color: {self.column_text_color};'>\n")
        for i in range(0, len(self.columns)):
            file.write(
                f"<td align='{self.cell_horizontal_text_align}'>{self.columns[i]}</td>\n")
        file.write(f"</tr>\n")

        for i in range(0, len(self.data)):
            file.write(
                f"<tr Valign='{self.cell_vertical_text_align}' bgcolor='{self.data_bg_color}' style='color: {self.data_text_color};'>\n")
            for j in range(0, len(self.data[i])):
                file.write(
                    f"<td align='{self.cell_horizontal_text_align}'>{self.data[i][j]}</td>\n")
            file.write(f"</tr>\n")
        file.write(f"</table>\n</center>\n</body>\n")
        file.write('<script>\nfunction AutoRefresh(t){\nsetInterval("location.reload(true);",' +
                   str(self.update_time) + ')\n}\n</script>')
        file.write("\n</html>")
        file.close()
        if self.print_log:
            print(f"'{self.title}': Data Updated!")

    def open(self):
        """
        Function to open the table automatically.
        """
        if self.print_log:
            print(f"'{self.title}': Opening Table.....")
        os.system(f'start {self.location}.html')
        if self.print_log:
            print(f"'{self.title}': Table Opened!")

    def print_settings(self):
        print(f"Table '{self.title}' settings:-")
        print(
            f"\tTable Name:- {self.title}\n\tTable Location:- {self.location}.html\n\tTable Border:- {self.table_border}\n\tBorder Color:- {self.border_color}\n\tCell padding:- {self.cell_padding}\n\tCell Spacing:- {self.cell_spacing}\n\tHorizontal Text Align:- {self.cell_horizontal_text_align}\n\tVertical Text Align:- {self.cell_vertical_text_align}\n\tColumn Background Color:- {self.column_bg_color}\n\tData Background Color:- {self.data_bg_color}\n\tColumn Font Color:- {self.column_text_color}\n\tData Font Color:- {self.data_text_color}\n\tPrint Log:- {self.print_log}")
